﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Imaging;
using AdrianKousz.Util;

namespace AdrianKousz.GenieEngine
{
	public static class BitmapHelpers
	{
		public static unsafe void FillMap(Scenario scn, Bitmap bmp1, Bitmap bmp2)
		{
			using (var ubmp1 = new UnsafeBitmap(bmp1))
			using (var ubmp2 = new UnsafeBitmap(bmp2))
			{
				var factory = new ScnDefaultFactory();

				EnsureValid(ubmp1, ubmp2);

				var ptr1 = ubmp1.Scan0;
				var last1 = ptr1 + ubmp1.Stride * ubmp1.Height;

				var ptr2 = ubmp2.Scan0;
				var last2 = ptr2 + ubmp2.Stride * ubmp2.Height;

				var map = factory.MakeMap();
				map.SizeX = ubmp1.Width;
				map.SizeY = ubmp1.Height;
				map.LinearTiles = new Scenario.ScnMap.Tile[map.SizeX * map.SizeY];

				var gaiaunits = new List<Scenario.ScnUnit>();

				var treemapping = new short[256];
				treemapping[10] = 411;
				treemapping[13] = 351;
				treemapping[17] = 414;
				treemapping[18] = 348;
				treemapping[19] = 350;
				treemapping[20] = 349;

				var i = 0;
				var id = 0;
				while (ptr1 < last1 && ptr2 < last2) {
					var treeid = treemapping[*ptr1];
					if (treeid != 0) {
						var unit = factory.MakeUnit();
						var posy = i / map.SizeX;
						var posx = i - (posy * map.SizeX);
						unit.Id = id++;
						unit.UnitId = treeid;
						unit.PosX = posx + 0.5f;
						unit.PosY = posy + 0.5f;
						unit.Rotation = unit.InitialFrame = (short)Util.Math.Rand(0, 14);
						gaiaunits.Add(unit);
					}
					map.LinearTiles[i++] = new Scenario.ScnMap.Tile(*ptr1++, (byte)(*ptr2++ / 40), 0);
				}

				scn.NextId = id;
				scn.Map = map;
				scn.Units[0] = gaiaunits;
			}
		}

		private static void EnsureValid(UnsafeBitmap ubmp1, UnsafeBitmap ubmp2)
		{
			if (ubmp1.Width != ubmp2.Width || ubmp1.Height != ubmp2.Height)
				throw new ArgumentException("Bitmaps have unequal dimensions");
			Check(ubmp1);
			Check(ubmp2);
		}

		private static void Check(UnsafeBitmap ubmp)
		{
			if (ubmp.PixelFormat != PixelFormat.Format8bppIndexed)
				throw new ArgumentException("Unsupported PixelFormat");
			if (ubmp.Stride != ubmp.Width)
				throw new NotImplementedException("bmpbit.Stride != bmpbit.Width");
		}
	}
}

