﻿using System.IO;
using AdrianKousz.Util;

namespace AdrianKousz.GenieEngine
{
	public class ConvertCommand : CmdApp.Command
	{
		override public void Run(string[] args)
		{
			using (var srcstream = File.OpenRead(args[0]))
			using (var outstream = File.OpenWrite(args[1]))
			using (var refstream = File.OpenRead(args[2]))
			{
				var refscn = GenieFile.Deserialize<Scenario>(refstream);
				var srcscn = GenieFile.Deserialize<Scenario>(srcstream);

				refscn.Filename = srcscn.Filename;
				refscn.InstructionsUncompressed = srcscn.InstructionsUncompressed;
				refscn.NextId = srcscn.NextId;
				refscn.PlayerCount = srcscn.PlayerCount;

				refscn.PlayerSettings = srcscn.PlayerSettings;
				refscn.Resources2 = srcscn.Resources2;
				refscn.Messages = srcscn.Messages;
				refscn.Cinematics = srcscn.Cinematics;
				refscn.GlobalVictory = srcscn.GlobalVictory;

				refscn.Map = srcscn.Map;
				refscn.Units = srcscn.Units;

				refscn.PlayerSettings.ForEach(x => {
					x.NameStringID = -1;
					x.FilenameAI = "RandomGame"; x.FilenameCTY = ""; x.FilenamePER = "";
					x.ScriptAI = ""; x.ScriptCTY = ""; x.ScriptPER = "";
				});

				Enumerables.ForBoth(refscn.PlayerSettings2, srcscn.PlayerSettings2, (r, s) => {
					r.Name = s.Name;
					r.CameraX = s.CameraX;
					r.CameraY = s.CameraY;
					r.AlliedVictory = s.AlliedVictory;
					r.Stances = s.Stances;
				});

				var converter = new ConvertCommandHelper();
				converter.Convert(refscn);

				GenieFile.Serialize(refscn, outstream);
			}
		}

		public ConvertCommand()
		{
			MinArgumentCount = 3;
			Description = "Convert AOE1 scenario";
			Help = "Convert AOE1 scenario to AOE2"
				+ "\n"
				+ "\nThe converter needs an existing AOE2 scenario to start with."
				+ "\n"
				+ "\nParameters:"
				+ "\n<AOE1 scenario> <output scenario> <AOE2 reference scenario>"
			;
		}
	}
}
