﻿using System;
using System.Collections.Generic;
using AdrianKousz.Util;

namespace AdrianKousz.GenieEngine
{
	public class ConvertCommandHelper
	{
		public void Convert(Scenario scn)
		{
			ChangeUnits_AOE1_AOE2(scn);
			MoveUnits_AOE1_AOE2(scn);
			ChangeTiles_AOE1_AOE2(scn);
			RandomizeTrees_AOE2(scn);
		}

		/**
		 * Simple mapping of unit IDs
		 */
		private void ChangeUnits_AOE1_AOE2(Scenario scn)
		{
			var mapping = new short[] {
				// Resources
				66, 66,
				59, 59,
				102, 102,

				// Units
				83, 83,
				125, 125,

				// Buildings
				72, 72,
				117, 117,
				155, 155,
				79, 598,
				199, 79,
				69, 79,
				278, 79,
				70, 70,
				159, 285,
				109, 109,
				12, 12,
				68, 68,
				103, 562,
				101, 101,

				// Cliffs
				264, 264,
				265, 265,
				266, 266,
				267, 267,
				268, 268,
				269, 269,
				270, 270,
				271, 271,
				272, 272,
				273, 273,

				// Trees
				141, 349,
				144, 349,
				145, 349,
				146, 349,
				197, 350,
				194, 350,
				193, 350,
				192, 350,
				135, 350,
				137, 350,
				138, 350,
				139, 350,
				161, 350,
				198, 350,
				203, 350,
				226, 350,
				391, 350,
				392, 350,
				150, 351,
				151, 351,
				152, 351,
				153, 351,
				134, 411,
				140, 411,
				148, 411,
				136, 411,
				142, 411,
				143, 411,
				147, 411,
				195, 411,
				365, 411,
				366, 411,
				367, 411,
				393, 411,

				// Fish
				260, 69,
				263, 69,
				53, 456,
				52, 457,
				370, 451,

				// Animals
				48, 48,
				90, 48,
				65, 65,
				384, 65,
				126, 126,
				89, 126,
				96, 96,

				// Keep and process later
				50, 50,
				113, 113,
				114, 114,
				121, 121,
				129, 129,
			}.ToDictionary();

			scn.Units = scn.Units.Map(list => {
				return (IList<Scenario.ScnUnit>)list
					.Filter(unit => mapping.ContainsKey(unit.UnitId))
					.Map(unit => {
						unit.UnitId = mapping[unit.UnitId];
						return unit;
					})
					.ToList();
			}).ToList();
		}

		/**
		 * Some buildings are not the same size.
		 * They need to be moved by half a unit.
		 */
		private void MoveUnits_AOE1_AOE2(Scenario scn)
		{
			var minus = new int[] { 79, 68, 562, };
			var plus = new int[] { 109, };
			scn.Units.ForEach(list => {
				list.ForEach(unit => {
					minus.ForEach(buildingid => { if (unit.UnitId != buildingid) return; unit.PosX -= 0.5f; unit.PosY -= 0.5f; });
					plus.ForEach(buildingid => { if (unit.UnitId != buildingid) return; unit.PosX += 0.5f; unit.PosY += 0.5f; });
				});
			});
		}

		/**
		 * Farms need to be converted including the underlying terrain.
		 */
		private void ChangeTiles_AOE1_AOE2(Scenario scn)
		{
			var map = scn.Map;
			scn.Units.ForEach(list => {
				list.ForEach(unit => {
					var posx = (int)unit.PosX;
					var posy = (int)unit.PosY;
					var linearpos = posy * map.SizeX + posx;
					if (false) {
						// switch
					} else if (unit.UnitId == 50) {
						// Farm
						map.LinearTiles[linearpos].Id = 7;
						map.LinearTiles[linearpos - 1].Id = 7;
						map.LinearTiles[linearpos + 1].Id = 7;
						map.LinearTiles[linearpos - map.SizeX].Id = 7;
						map.LinearTiles[linearpos - map.SizeX - 1].Id = 7;
						map.LinearTiles[linearpos - map.SizeX + 1].Id = 7;
						map.LinearTiles[linearpos + map.SizeX].Id = 7;
						map.LinearTiles[linearpos + map.SizeX - 1].Id = 7;
						map.LinearTiles[linearpos + map.SizeX + 1].Id = 7;
					} else if (unit.UnitId == 113 || unit.UnitId == 114 || unit.UnitId == 121 || unit.UnitId == 129) {
						if (map.LinearTiles[linearpos].Id == 20) {
							map.LinearTiles[linearpos].Id = 17;
							unit.UnitId = 414;
						} else {
							unit.UnitId = 351;
						}
					}
				});
			});
		}

		/**
		 * Forests would contain the same tree graphic
		 * without this function
		 */
		private void RandomizeTrees_AOE2(Scenario scn)
		{
			var treeids = new int[] {
				411, 351, 414, 350, 348, 349,
			};
			scn.Units.ForEach(list => {
				list.ForEach(unit => {
					treeids.ForEach(treeid => {
						if (unit.UnitId == treeid) {
							unit.Rotation = unit.InitialFrame = (short)Util.Math.Rand(0, 14);
						}
					});
				});
			});
		}
	}
}
